#pragma pack_matrix(column_major)
#pragma vertex vert
#pragma fragment frag
#pragma pc_length 112

struct VertexInput
{
    float3 Position : POSITION;
    float3 Normal : NORMAL;
    float2 UV0 : TEXCOORD0;
};

struct VertexOutput
{
    float4 Position : SV_Position;
    float3 Normal : TEXCOORD0;
    float2 UV0 : TEXCOORD1;
};

struct PushConstants
{
    float4x4 matrix_mvp;
    float4 objectColour;
    float4 mainLight;
    float4 ambientLight;
};

[[vk::push_constant]]
PushConstants pc;

VertexOutput vert(VertexInput input)
{
    VertexOutput output;
    
    output.Position = mul(pc.matrix_mvp, float4(input.Position, 1));
    output.Normal = input.Normal;
    output.UV0 = input.UV0;
    
    return output;
}

float4 frag(VertexOutput v) : SV_Target
{
    return pc.objectColour * (saturate(dot(v.Normal, pc.mainLight.xyz)) + pc.ambientLight);
}